<?php
/**
 * PDF Invoicing for WooCommerce - Core Class
 *
 * @version 1.1.1
 * @since   1.0.0
 * @author  Algoritmika Ltd
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_PDF_Invoicing_Core' ) ) :

class Alg_WC_PDF_Invoicing_Core {

	/**
	 * Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] order meta box
	 * @todo    [dev] (maybe) replace TCPDF lib with `tc-lib-pdf`
	 */
	function __construct() {
		// Action
		do_action( 'alg_wc_pdf_invoicing_before_core_loaded', $this );
		// Functions
		require_once( 'functions/alg-wc-pdf-invoicing-functions.php' );
		require_once( 'functions/alg-wc-pdf-invoicing-functions-default-values.php' );
		// Classes
		require_once( 'classes/class-alg-wc-pdf-invoicing-doc.php' );
		// Core
		if ( 'yes' === get_option( 'alg_wc_pdf_invoicing_plugin_enabled', 'yes' ) ) {
			// Hooks
			add_action( 'admin_init', array( $this, 'delete_doc' ) );
			add_action( 'admin_init', array( $this, 'view_doc' ) );
			add_action( 'admin_init', array( $this, 'create_doc' ) );
			// Create docs automatically
			$all_hooks = array();
			foreach ( apply_filters( 'alg_wc_pdf_invoicing_enabled_docs', array( '0' ) ) as $doc_id ) {
				$doc_hooks = $this->get_doc_option( $doc_id, 'hooks' );
				$all_hooks = array_merge( $all_hooks, $doc_hooks );
			}
			$all_hooks = array_unique( $all_hooks );
			foreach ( $all_hooks as $hook ) {
				if ( 'manual' != $hook ) {
					add_action( $hook, array( $this, 'create_docs' ) );
				}
			}
			// Emails
			add_filter( 'woocommerce_email_attachments', array( $this, 'email_attachments' ), PHP_INT_MAX, 4 );
			// Shortcodes
			$this->shortcodes = require_once( 'class-alg-wc-pdf-invoicing-shortcodes.php' );
		}
	}

	/**
	 * do_shortcode.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function do_shortcode( $value, $doc_obj ) {
		$this->shortcodes->set_doc_obj( $doc_obj );
		return do_shortcode( $value );
	}

	/**
	 * get_doc_option.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_doc_option( $doc_id, $option_id, $do_shortcode = false, $doc_obj = null ) {
		if ( ! isset( $this->doc_options[ $doc_id ] ) ) {
			$this->doc_options[ $doc_id ] = get_option( "alg_wc_pdf_invoicing_doc_{$doc_id}", array() );
		}
		$value = ( isset( $this->doc_options[ $doc_id ][ $option_id ] ) ? $this->doc_options[ $doc_id ][ $option_id ] : alg_wc_pdf_invoicing_get_default( $option_id, $doc_id ) );
		return ( $do_shortcode ? $this->do_shortcode( $value, $doc_obj ) : $value );
	}

	/**
	 * get_and_update_counter.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] (maybe) re-test MySQL transaction
	 * @todo    [dev] (maybe) need another solution (instead of `get_option()`): a) new table (with auto-incremental counter); b) save counter as custom post (i.e. template) option
	 */
	function get_and_update_counter( $doc_id ) {
		global $wpdb;
		$doc_id = 'doc_' . $doc_id;
		$wpdb->query( 'START TRANSACTION' );
		$counters            = get_option( 'alg_wc_pdf_invoicing_counters', array() );
		$counter             = ( isset( $counters[ $doc_id ] ) ? $counters[ $doc_id ] : 1 );
		$counters[ $doc_id ] = $counter + 1;
		$result              = update_option( 'alg_wc_pdf_invoicing_counters', $counters );
		$wpdb->query( ( $result ? 'COMMIT' : 'ROLLBACK' ) );
		return $counter;
	}

	/**
	 * email_attachments.
	 *
	 * @version 1.1.1
	 * @since   1.0.0
	 * @todo    [dev] per payment gateways
	 */
	function email_attachments( $attachments, $email_id, $email_order, $email = false ) {
		if ( is_object( $email_order ) && is_a( $email_order, 'WC_Order' ) ) {
			foreach ( apply_filters( 'alg_wc_pdf_invoicing_enabled_docs', array( '0' ) ) as $doc_id ) {
				$doc_emails = $this->get_doc_option( $doc_id, 'emails' );
				if ( in_array( $email_id, $doc_emails ) ) {
					if ( ! isset( $order_id ) ) {
						$order_id = $email_order->get_id();
					}
					$doc = new Alg_WC_PDF_Invoicing_Doc( $order_id, $doc_id );
					if ( $doc->is_created() ) {
						$tmp_dir = trailingslashit( get_temp_dir() );
						$doc->get_pdf( 'F', $tmp_dir );
						$attachments[] = $tmp_dir . $doc->get_number() . '.pdf';
					}
				}
			}
		}
		return $attachments;
	}

	/**
	 * create_docs.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] per payment gateways
	 * @todo    [dev] optionally do not create when order total is zero
	 */
	function create_docs( $order_id ) {
		foreach ( apply_filters( 'alg_wc_pdf_invoicing_enabled_docs', array( '0' ) ) as $doc_id ) {
			$doc_hooks = $this->get_doc_option( $doc_id, 'hooks' );
			if ( in_array( current_filter(), $doc_hooks ) ) {
				$doc = new Alg_WC_PDF_Invoicing_Doc( $order_id, $doc_id );
				if ( ! $doc->is_created() ) {
					$doc->create( array(
						'author' => 0,
						'id'     => $this->get_and_update_counter( $doc_id ),
						'date'   => time(),
					) );
				}
			}
		}
	}

	/**
	 * create_doc.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] add notice
	 * @todo    [dev] (maybe) check if `manual` is enabled (or `hooks` option is empty)
	 */
	function create_doc() {
		if ( isset( $_GET['alg-wc-pdf-invoicing-create-doc'] ) ) {
			if ( ! current_user_can( 'manage_woocommerce' ) ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Insufficient user capability.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			if ( ! isset( $_GET['alg-wc-pdf-invoicing-order-id'] ) ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Order ID not set.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			$doc_id   = intval( $_GET['alg-wc-pdf-invoicing-create-doc'] );
			$order_id = intval( $_GET['alg-wc-pdf-invoicing-order-id'] );
			$doc      = new Alg_WC_PDF_Invoicing_Doc( $order_id, $doc_id );
			if ( $doc->is_created() ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Document is already created.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			$doc->create( array(
				'author' => get_current_user_id(),
				'id'     => $this->get_and_update_counter( $doc_id ),
				'date'   => time(),
			) );
			wp_safe_redirect( remove_query_arg( array( 'alg-wc-pdf-invoicing-create-doc', 'alg-wc-pdf-invoicing-order-id' ) ) );
			exit;
		}
	}

	/**
	 * view_doc.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function view_doc() {
		if ( isset( $_GET['alg-wc-pdf-invoicing-view-doc'] ) ) {
			if ( ! current_user_can( 'manage_woocommerce' ) ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Insufficient user capability.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			if ( ! isset( $_GET['alg-wc-pdf-invoicing-order-id'] ) ) {
				echo sprintf( __( 'Error: %s.', 'pdf-invoicing-for-woocommerce' ), __( 'Order ID not set', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			$doc_id   = intval( $_GET['alg-wc-pdf-invoicing-view-doc'] );
			$order_id = intval( $_GET['alg-wc-pdf-invoicing-order-id'] );
			$doc      = new Alg_WC_PDF_Invoicing_Doc( $order_id, $doc_id );
			if ( ! $doc->is_created() ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Document is not created yet.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			echo $doc->get_pdf( ( isset( $_GET['alg-wc-pdf-invoicing-pdf-dest'] ) ? wc_clean( $_GET['alg-wc-pdf-invoicing-pdf-dest'] ) : 'I' ) );
			die();
		}
	}

	/**
	 * delete_doc.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] add notice
	 * @todo    [dev] make optional (e.g. visible to admin but not for shop manager etc.)
	 */
	function delete_doc() {
		if ( isset( $_GET['alg-wc-pdf-invoicing-delete-doc'] ) ) {
			if ( ! current_user_can( 'manage_woocommerce' ) ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Insufficient user capability.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			if ( ! isset( $_GET['alg-wc-pdf-invoicing-order-id'] ) ) {
				echo sprintf( __( 'Error: %s.', 'pdf-invoicing-for-woocommerce' ), __( 'Order ID not set', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			$doc_id   = intval( $_GET['alg-wc-pdf-invoicing-delete-doc'] );
			$order_id = intval( $_GET['alg-wc-pdf-invoicing-order-id'] );
			$doc      = new Alg_WC_PDF_Invoicing_Doc( $order_id, $doc_id );
			if ( ! $doc->is_created() ) {
				echo sprintf( __( 'Error: %s', 'pdf-invoicing-for-woocommerce' ), __( 'Document is not created yet.', 'pdf-invoicing-for-woocommerce' ) );
				die();
			}
			$doc->remove();
			wp_safe_redirect( remove_query_arg( array( 'alg-wc-pdf-invoicing-delete-doc', 'alg-wc-pdf-invoicing-order-id' ) ) );
			exit;
		}
	}

}

endif;

return new Alg_WC_PDF_Invoicing_Core();
