<?php
/**
 * PDF Invoicing for WooCommerce - Doc Class
 *
 * @version 1.1.0
 * @since   1.0.0
 * @author  Algoritmika Ltd
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_PDF_Invoicing_Doc' ) ) :

class Alg_WC_PDF_Invoicing_Doc {

	/**
	 * Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function __construct( $order_id, $doc_id ) {
		$this->order_id = $order_id;
		$this->doc_id   = $doc_id;
		$data           = get_post_meta( $this->order_id, '_alg_wc_pdf_invoicing_data', true );
		$this->data     = ( ! empty( $data[ $this->doc_id ] ) ? $data[ $this->doc_id ] : false );
	}

	/**
	 * is_created.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function is_created() {
		return ( false !== $this->data );
	}

	/**
	 * create.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] (maybe) also save prefix and other data
	 * @todo    [dev] (maybe) also save as a separate order meta, e.g. `_alg_wc_pdf_invoicing_number`, `_alg_wc_pdf_invoicing_date`
	 */
	function create( $data ) {
		$data_all_docs = get_post_meta( $this->order_id, '_alg_wc_pdf_invoicing_data', true );
		if ( empty( $data_all_docs ) ) {
			$data_all_docs = array();
		}
		$data_all_docs[ $this->doc_id ] = $data;
		update_post_meta( $this->order_id, '_alg_wc_pdf_invoicing_data', $data_all_docs );
		$this->data = $data;
	}

	/**
	 * remove.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] (maybe) better function name, e.g. `delete`, `destroy`, `trash`
	 */
	function remove() {
		$data_all_docs = get_post_meta( $this->order_id, '_alg_wc_pdf_invoicing_data', true );
		if ( empty( $data_all_docs ) ) {
			$data_all_docs = array();
		}
		$data_all_docs[ $this->doc_id ] = false;
		update_post_meta( $this->order_id, '_alg_wc_pdf_invoicing_data', $data_all_docs );
		$this->data = false;
	}

	/**
	 * get_counter.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_counter() {
		return $this->data['id'];
	}

	/**
	 * get_number.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_number() {
		alg_wc_pdf_invoicing()->core->shortcodes->set_prop( 'doc_nr', false );
		$return = $this->get_doc_option( 'number_format', true );
		alg_wc_pdf_invoicing()->core->shortcodes->set_prop( 'doc_nr', true );
		return $return;
	}

	/**
	 * get_formatted_date.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_formatted_date() {
		return date_i18n( 'Y-m-d', $this->data['date'] );
	}

	/**
	 * get_author_full_name.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_author_full_name() {
		$doc_author = $this->data['author'];
		$doc_author = get_userdata( $doc_author );
		return ( $doc_author ? $doc_author->first_name . ' ' . $doc_author->last_name : '' );
	}

	/**
	 * get_doc_option.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_doc_option( $option_id, $do_shortcode = false ) {
		return alg_wc_pdf_invoicing()->core->get_doc_option( $this->doc_id, $option_id, $do_shortcode, $this );
	}

	/**
	 * get_html.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_html() {
		return '<meta charset="UTF-8">' . '<style>' . $this->get_doc_option( 'html_style', true ) . '</style>' . $this->get_doc_option( 'html_content', true );
	}

	/**
	 * get_pdf.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 * @see     https://tcpdf.org/
	 * @todo    [dev] more customizable options (e.g. fonts etc.)
	 * @todo    [dev] custom size page format
	 * @todo    [dev] (maybe) `$pdf->setFontSubsetting( true );` (before `$pdf->SetFont()`)
	 */
	function get_pdf( $dest = 'I', $path = '' ) {
		$uploads_dir = wp_upload_dir();
		define( 'K_PATH_IMAGES', $uploads_dir['basedir'] . '/' );
		require_once( alg_wc_pdf_invoicing()->plugin_path() . '/assets/lib/tcpdf/tcpdf.php' );
		require_once( 'class-alg-wc-pdf-invoicing-tcpdf.php' );
		$pdf = new Alg_WC_PDF_Invoicing_TCPDF(
			$this->get_doc_option( 'page_orientation' ),
			'mm',
			$this->get_doc_option( 'page_format' ),
			true,
			'UTF-8',
			false
		);
		$pdf->alg_wc_pdf_invoicing_doc = $this;
		$pdf->SetCreator( PDF_CREATOR);
		$pdf->SetAuthor( 'PDF Invoicing for WooCommerce' );
		$pdf->SetTitle( $this->get_number() );
		$pdf->SetHeaderData( $this->get_doc_option( 'header_img', true ), 50, $this->get_doc_option( 'header_title', true ), $this->get_doc_option( 'header_text', true ) );
		$pdf->setHeaderFont( array( 'dejavusans', '', $this->get_doc_option( 'header_font_size' ) ) );
		$pdf->setFooterFont( array( 'dejavusans', '', $this->get_doc_option( 'footer_font_size' ) ) );
		$pdf->SetDefaultMonospacedFont( PDF_FONT_MONOSPACED );
		$pdf->SetMargins( PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT );
		$pdf->SetHeaderMargin( PDF_MARGIN_HEADER );
		$pdf->SetFooterMargin( PDF_MARGIN_FOOTER );
		$pdf->SetAutoPageBreak( TRUE, PDF_MARGIN_BOTTOM );
		$pdf->setImageScale( PDF_IMAGE_SCALE_RATIO );
		$pdf->SetFont( 'dejavusans', '', $this->get_doc_option( 'font_size' ), '', true );
		$pdf->AddPage();
		$pdf->writeHTML( $this->get_html(), true, false, true, false, '' );
		$pdf->Output( $path . $this->get_number() . '.pdf', $dest );
	}
}

endif;
